{
  description = "Collection of DSLs which can solve discrete mathematics problems";

  inputs = {
    nixpkgs.url = "nixpkgs/nixos-21.11";

    flake-utils = {
      url = "github:numtide/flake-utils";
      inputs.nixpkgs.follows = "nixpkgs";
    };
  };
  outputs = { self, flake-utils, nixpkgs }:
  flake-utils.lib.eachSystem [
    "aarch64-linux"
    "aarch64-darwin"
    "i686-linux"
    "x86_64-darwin"
    "x86_64-linux"
  ]
  (system: let
    pkgs = import nixpkgs { inherit system; }; 
    cargoTOML = pkgs.lib.importTOML ./Cargo.toml;
    version = cargoToml.package.version;
  in rec {

    defaultApp = apps.diskmat-tools;

    apps.diskmat-tools = {
      type = "app";
      program = "${packages.diskmat-tools}/bin/diskmat_tools";
    };

    defaultPackage = packages.diskmat-tools;

    packages = {

      diskmat-tools = pkgs.rustPlatform.buildRustPackage {
        pname = "diskmat-tools";
        inherit version;
        src = ./.;
    
        buildInputs = [ pkgs.graphviz ];
        cargoLock.lockFile = ./Cargo.lock;
      };

      diskmat-tools-examples = pkgs.stdenv.mkDerivation {
        pname = "diskmat-tools-examples";
        inherit version;
        src = ./example_files;
        nativeBuildInputs = [ packages.diskmat-tools ];

        # TODO: `rm -r matrix` should be removed after support is implemented
        buildPhase = ''
          rm -r matrix
          for file in $(find -type f)
          do
            diskmat_tools -i $file -o "''${file%.*}.png"
          done
        '';

        installPhase = ''
          mkdir $out
          cp -r . $out
        '';

      };

    };
  });
}
